/***************************************************************************
 *   Copyright (C) 2005 by Ken Werner                                      *
 *   ken.werner@web.de                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "flowlayout.h"
//#include "kdebug.h"

class FlowLayoutIterator :public QGLayoutIterator{
public:
	FlowLayoutIterator( QPtrList<QLayoutItem>* layoutItems ): 
		index(0),
		mLayoutItems(layoutItems){
	}
	uint count() const;
	QLayoutItem* current();
	QLayoutItem* next();
	QLayoutItem* takeCurrent();
private:
	int index;
	QPtrList<QLayoutItem>* mLayoutItems;
};
QLayoutItem* FlowLayoutIterator::current(){
	return index < int(mLayoutItems->count()) ? mLayoutItems->at(index) : 0;
}
QLayoutItem* FlowLayoutIterator::next(){
	index++; 
	return current();
}
QLayoutItem* FlowLayoutIterator::takeCurrent(){
	return index < int(mLayoutItems->count()) ? mLayoutItems->take(index) : 0;
}


FlowLayout::FlowLayout( QWidget* parent, Qt::Orientation orientation, int border, int space, const char* name )
	: QLayout( parent, border, space, name ),
	mOrientation(orientation){
}
FlowLayout::FlowLayout( QLayout* parent, Qt::Orientation orientation, int space, const char* name )
	: QLayout( parent, space, name ),
	mOrientation(orientation){
}
FlowLayout::FlowLayout( Qt::Orientation orientation, int space, const char* name )
	: QLayout( space, name ),
	mOrientation(orientation){
}

FlowLayout::~FlowLayout(){
	deleteAllItems();
}

int FlowLayout::heightForWidth( int w ) const{
	FlowLayout* mthis = (FlowLayout*)this;
	int h = mthis->doLayout( QRect(0,0,w,0), TRUE );
	return h;
}

int FlowLayout::widthForHeight( int h ) const{
	FlowLayout* mthis = (FlowLayout*)this;
	int w = mthis->doLayout( QRect(0,0,0,h), TRUE );
	return w;
}

void FlowLayout::addItem( QLayoutItem* item){
	item->widget()->show();
	mLayoutItems.append( item );
}

void FlowLayout::remove(QWidget* widget){
	widget->hide();
	QLayout::remove(widget); // removes and deletes only the QLayoutItem (QWidgetItem)
}

bool FlowLayout::hasHeightForWidth() const{
	if(mOrientation == Qt::Horizontal)
		return FALSE;
	else
		return TRUE;
}

bool FlowLayout::hasWidthForHeight() const{
	if(mOrientation == Qt::Horizontal)
		return TRUE;
	else
		return FALSE;
}

QSize FlowLayout::sizeHint() const{
	//return minimumSize();
	QSize size(0,0);
	QPtrListIterator<QLayoutItem> it(mLayoutItems);
	QLayoutItem *o;
	while((o=it.current()) != 0){
		++it;
		size = size.expandedTo( o->sizeHint() );
	}
	return size;
}

QSize FlowLayout::minimumSize() const{
	QSize size(0,0);
	QPtrListIterator<QLayoutItem> it(mLayoutItems);
	QLayoutItem *o;
	while((o=it.current()) != 0){
		++it;
		size = size.expandedTo(o->minimumSize());
	}
	return size;
}

QSizePolicy::ExpandData FlowLayout::expanding() const{
	return QSizePolicy::NoDirection;
}

QLayoutIterator FlowLayout::iterator(){
	return QLayoutIterator( new FlowLayoutIterator( &mLayoutItems ) );
}

Qt::Orientation FlowLayout::getOrientation() const{
	return mOrientation;
}

void FlowLayout::setOrientation(Qt::Orientation orientation){
	mOrientation = orientation;
}

void FlowLayout::setGeometry( const QRect& rect ){
	QLayout::setGeometry( rect );
	doLayout( rect );
}

int FlowLayout::doLayout( const QRect& rect, bool testonly ){
	if(mOrientation == Qt::Horizontal)
		return doLayoutHorizontal(rect, testonly);
	else
		return doLayoutVertical(rect, testonly);
}

int FlowLayout::doLayoutHorizontal( const QRect& rect, bool testOnly ){
	//kdDebug() << "spacing: " << spacing() <<  endl;

	int x = rect.x();
	int y = rect.y();
	int width = 0; // width of this column so far
	int height = 0; // height of this column so far
	QPtrListIterator<QLayoutItem> it(mLayoutItems);
	QLayoutItem* layoutItem;
	QPtrList<QLayoutItem> column; // stores the items of one column
	while((layoutItem = it.current()) != 0){
		++it;
		//int nextY = y + layoutItem->sizeHint().height() + spacing(); // next y
		int nextY = y + layoutItem->sizeHint().height(); // next y
		//if( nextY - spacing() > rect.bottom() && width > 0 ) {
		if( nextY > rect.bottom() && width > 0 ) {
			// next column
			y = rect.y(); // reset y
			x = x + width + spacing(); // new x
			//nextY = y + layoutItem->sizeHint().height() + spacing(); // next y with changed y
			nextY = y + layoutItem->sizeHint().height(); // next y with changed y
			width = 0; // reset width for the next column
		}
		if(!testOnly){
			layoutItem->setGeometry( QRect( QPoint( x, y ), layoutItem->sizeHint() ) );
			column.append(layoutItem);
			height += layoutItem->sizeHint().height(); // add the height of the current item to the column height
			if( it.current() == 0 || nextY + it.current()->sizeHint().height() > rect.bottom() ){ // test it it's the last item (of this column)
				// calculate real needed width
				int rWidth = 0;
				for(QLayoutItem* item = column.first(); item; item = column.next()){
					rWidth = QMAX( rWidth, item->widget()->sizeHint().width() );
				}
				// relayout the items of the former column
				int space = (rect.height() - height) / (column.count() + 1);
				int i = 0; // counts the items of this column
				for(QLayoutItem* item = column.first(); item; item = column.next()){
					QRect r = item->geometry();
					item->setGeometry( QRect(r.left(), r.top() + ((++i) * space), rWidth, r.height()) );
				}
				column.clear(); // remove the items of the former column
				height = 0; // reset height for the next column
			}
		}
		y = nextY;
		width = QMAX( width, layoutItem->sizeHint().width() );
	}
	return x + width - rect.x(); // width
}

int FlowLayout::doLayoutVertical( const QRect& rect, bool testOnly ){
	int x = rect.x();
	int y = rect.y();
	int height = 0; // height of this line so far
	QPtrListIterator<QLayoutItem> it(mLayoutItems);
	QLayoutItem* layoutItem;
	while((layoutItem = it.current() ) != 0){
		++it;
		//int nextX = x + layoutItem->sizeHint().width() + spacing();
		int nextX = x + layoutItem->sizeHint().width();
		if(nextX - spacing() > rect.right() && height > 0) {
			// next line
			x = rect.x(); // reset x
			//y = y + height + spacing(); // new y
			y = y + height; // new y
			//nextX = x + layoutItem->sizeHint().width() + spacing(); // next x
			nextX = x + layoutItem->sizeHint().width(); // next x
			height = 0; // reset height for the next line
		}
		const int itemHeight = layoutItem->sizeHint().height();
		if(!testOnly)
			layoutItem->setGeometry(QRect(x, y, rect.right(), itemHeight));
		x = nextX;
		height = QMAX(height, itemHeight);
	}
	return y + height - rect.y(); // height
}
