/***************************************************************************
 *   Copyright (C) 2006 by Ken Werner                                      *
 *   ken.werner@web.de                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "defaultsource.h"
#include <qlabel.h>
#include <math.h>
#include <kcolorbutton.h>
#include <kfontrequester.h>
#include <qcombobox.h>
#include <qcheckbox.h>

//#include <kdebug.h>

DefaultSource::DefaultSource(QWidget* inParent, const QFile& inSourceFile):
		Source(inParent),
		mSourceFile(inSourceFile.name()){
	mDefaultSourcePrefs = 0; // the prefs dialog is created in the addPrefs method
	mLabel = new QLabel(inParent);
	mRefreshTimer = new QTimer(this, "default refresh handler" );
	// connect the timer
	connect( mRefreshTimer, SIGNAL(timeout()), this, SLOT(updateValues()) );
	// start the timer
	mRefreshTimer->start( 500 ); // 500 ms
}

QWidget* DefaultSource::getWidget(){
	return mLabel;
}

QWidget* DefaultSource::addPrefs(QWidget* inParent){
	if(!mDefaultSourcePrefs){
		mDefaultSourcePrefs = new DefaultSourcePrefs(inParent, "defaultsourceprefsui");
		mDefaultSourcePrefs->alignmentComboBox->insertItem("Left");
		mDefaultSourcePrefs->alignmentComboBox->insertItem("Center");
		mDefaultSourcePrefs->alignmentComboBox->insertItem("Right");

		// disable widgets if enable source checkbox is disabled
		QCheckBox* enabledCheckBox = ((SourcePrefs*)getPrefs())->enabledCheckBox;
		connect(enabledCheckBox, SIGNAL(toggled(bool)), mDefaultSourcePrefs->colorLabel, SLOT(setEnabled(bool)));
		connect(enabledCheckBox, SIGNAL(toggled(bool)), mDefaultSourcePrefs->colorButton, SLOT(setEnabled(bool)));
		connect(enabledCheckBox, SIGNAL(toggled(bool)), mDefaultSourcePrefs->fontLabel, SLOT(setEnabled(bool)));
		connect(enabledCheckBox, SIGNAL(toggled(bool)), mDefaultSourcePrefs->fontRequester, SLOT(setEnabled(bool)));
		connect(enabledCheckBox, SIGNAL(toggled(bool)), mDefaultSourcePrefs->alignmentLabel, SLOT(setEnabled(bool)));
		connect(enabledCheckBox, SIGNAL(toggled(bool)), mDefaultSourcePrefs->alignmentComboBox, SLOT(setEnabled(bool)));		
	}
	return mDefaultSourcePrefs;
}

void DefaultSource::updatePrefsGUI(){
	Source::updatePrefsGUI(); // update prefs of the super class
	mDefaultSourcePrefs->colorButton->setColor(mLabel->paletteForegroundColor());
	mDefaultSourcePrefs->fontRequester->setFont(mLabel->font());
	switch (mLabel->alignment()) {
		case Qt::AlignCenter:
			mDefaultSourcePrefs->alignmentComboBox->setCurrentItem(1);
			break;
		case Qt::AlignRight:
			mDefaultSourcePrefs->alignmentComboBox->setCurrentItem(2);
			break;
		default: // Qt::AlignLeft
			break;
			mDefaultSourcePrefs->alignmentComboBox->setCurrentItem(0);
	}
	// disable if source is disabled
	mDefaultSourcePrefs->colorLabel->setEnabled(mEnabled);
	mDefaultSourcePrefs->colorButton->setEnabled(mEnabled);
	mDefaultSourcePrefs->fontLabel->setEnabled(mEnabled);
	mDefaultSourcePrefs->fontRequester->setEnabled(mEnabled);
	mDefaultSourcePrefs->alignmentLabel->setEnabled(mEnabled);
	mDefaultSourcePrefs->alignmentComboBox->setEnabled(mEnabled);
}

void DefaultSource::applyPrefs(){
	Source::applyPrefs(); // call apply prefs of the super class
	mLabel->setPaletteForegroundColor(mDefaultSourcePrefs->colorButton->color());
	mLabel->setFont(mDefaultSourcePrefs->fontRequester->font());
	int alignID = mDefaultSourcePrefs->alignmentComboBox->currentItem();
	Qt::AlignmentFlags align = Qt::AlignCenter;
	if(alignID == 0){
		align = Qt::AlignLeft;
	}else if(alignID == 2){
		align = Qt::AlignRight;
	}
	mLabel->setAlignment(align);
}

void DefaultSource::savePrefs(KConfig* inKConfig){
	Source::savePrefs(inKConfig);
	inKConfig->writeEntry(mID + "_color", mDefaultSourcePrefs->colorButton->color());
	inKConfig->writeEntry(mID + "_font", mDefaultSourcePrefs->fontRequester->font());
	inKConfig->writeEntry(mID + "_align", mLabel->alignment());
}

void DefaultSource::loadPrefs(KConfig* inKConfig){
	Source::loadPrefs(inKConfig);
	QColor color = inKConfig->readColorEntry(mID + "_color");
	if(!color.isValid())
		color.setRgb (0,0,0);
	mLabel->setPaletteForegroundColor(color);
	mLabel->setFont(inKConfig->readFontEntry(mID + "_font"));
	mLabel->setAlignment(inKConfig->readNumEntry(mID + "_align"));
}

QString DefaultSource::getPath() const{
	return mSourceFile.name();
}

void DefaultSource::updateValues() {
	QString text = (getName().isEmpty() || !showName()) ? getValue() : getName() + ": " + getValue();
	if(mLabel->text() != text)
		mLabel->setText(text);
}

QString DefaultSource::KHzinHumanReadable( uint value ) const{
	if( value >= 1000000 )
		return QString::number( round(value/1000000.0, 1) ) + " GHz";
	else if( value >= 1000 )
		return QString::number( round(value/1000.0, -1) ) + " MHz";
	else
		return QString::number( value ) + " KHz";
}

double DefaultSource::round(double inValue, int inDigits) const{
    return floor(inValue * pow( 10, inDigits) + 0.5) * pow(10, -inDigits);
}

int DefaultSource::celsiusToFahrenheit(int inCelsius) const{
	return qRound(1.8 * inCelsius + 32);
}
